@@ -24,6 +24,7 @@
 package htsjdk.samtools;
 
 import htsjdk.samtools.SAMFileHeader.SortOrder;
+import htsjdk.samtools.SAMValidationError.Type;
 import htsjdk.samtools.util.DateParser;
 import htsjdk.samtools.util.LineReader;
 import htsjdk.samtools.util.RuntimeIOException;
@@ -55,14 +56,12 @@ public class SAMTextHeaderCodec {
     private String mSource;
     private List<SAMSequenceRecord> sequences;
     private List<SAMReadGroupRecord> readGroups;
-    // Accumulate header while reading it from input.
-    private final StringBuilder textHeader = new StringBuilder();
 
     // For error reporting when parsing
     private ValidationStringency validationStringency = ValidationStringency.SILENT;
 
     // These attributes are populated when generating text
-    private BufferedWriter writer;
+    private Writer writer;
 
     private static final String TAG_KEY_VALUE_SEPARATOR = ":";
     private static final char TAG_KEY_VALUE_SEPARATOR_CHAR = ':';
@@ -73,7 +72,7 @@ public class SAMTextHeaderCodec {
     public static final String COMMENT_PREFIX = HEADER_LINE_START + HeaderRecordType.CO.name() + FIELD_SEPARATOR;
     private static final Log log = Log.getInstance(SAMTextHeaderCodec.class);
 
-    void setWriter(final BufferedWriter writer) {
+    void setWriter(final Writer writer) {
         this.writer = writer;
     }
 
@@ -124,23 +123,14 @@ public class SAMTextHeaderCodec {
         mFileHeader.setSequenceDictionary(new SAMSequenceDictionary(sequences));
         mFileHeader.setReadGroups(readGroups);
 
-        // Only store the header text if there was a parsing error or the it's less than 1MB on disk / 2MB in mem
-        if (!mFileHeader.getValidationErrors().isEmpty() || textHeader.length() < (1024 * 1024)) {
-            mFileHeader.setTextHeader(textHeader.toString());
-        }
-
         SAMUtils.processValidationErrors(mFileHeader.getValidationErrors(), -1, validationStringency);
         return mFileHeader;
     }
 
     private String advanceLine() {
         final int nextChar = mReader.peek();
-        if (nextChar != '@') {
-            return null;
-        }
-        mCurrentLine = mReader.readLine();
-        textHeader.append(mCurrentLine).append('\n');
-        return mCurrentLine;
+        this.mCurrentLine = (nextChar == '@') ? mReader.readLine() : null;
+        return this.mCurrentLine;
     }
 
     /**
@@ -547,4 +537,4 @@ public class SAMTextHeaderCodec {
         }
         this.validationStringency = validationStringency;
     }
-}
\ No newline at end of file
+}
