@@ -85,6 +85,11 @@ public interface SamReader extends Iterable<SAMRecord>, Closeable {
         public static final Type CRAM_TYPE = new TypeImpl("CRAM", "cram", "crai");
         public static final Type BAM_TYPE = new TypeImpl("BAM", "bam", "bai");
         public static final Type SAM_TYPE = new TypeImpl("SAM", "sam", null);
+        public static final Type BAM_CSI_TYPE = new TypeImpl("BAM", "bam", "csi");
+
+        public boolean hasValidFileExtension(final String fileName) {
+            return fileName != null && fileName.endsWith("." + fileExtension());
+        }
     }
 
     /**
