@@ -35,7 +35,6 @@ import java.io.InputStream;
 import java.lang.ref.WeakReference;
 import java.net.URL;
 import java.nio.file.Path;
-import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -76,7 +75,6 @@ public class ReferenceSource implements CRAMReferenceSource {
      *
      * @return CRAMReferenceSource if one can be acquired. Guaranteed to not be null if none
      * of the listed exceptions is thrown.
-     * @throws IllegalStateException if no default reference source can be acquired
      * @throws IllegalArgumentException if the reference_fasta environment variable refers to a
      * a file that doesn't exist
      *<p>
@@ -91,6 +89,7 @@ public class ReferenceSource implements CRAMReferenceSource {
      public static CRAMReferenceSource getDefaultCRAMReferenceSource() {
         if (null != Defaults.REFERENCE_FASTA) {
             if (Defaults.REFERENCE_FASTA.exists()) {
+                log.info(String.format("Default reference file %s exists, so going to use that.", Defaults.REFERENCE_FASTA.getAbsolutePath()));
                 return new ReferenceSource(Defaults.REFERENCE_FASTA);
             }
             else {
@@ -99,18 +98,14 @@ public class ReferenceSource implements CRAMReferenceSource {
             }
         }
         else if (Defaults.USE_CRAM_REF_DOWNLOAD) {
+            log.info("USE_CRAM_REF_DOWNLOAD=true, so attempting to download reference file as needed.");
             return new ReferenceSource((ReferenceSequenceFile)null);
         }
         else {
-            throw new IllegalStateException(
-                    "A valid CRAM reference was not supplied and one cannot be acquired via the property settings reference_fasta or use_cram_ref_download");
+            return new CRAMLazyReferenceSource();
         }
     }
 
-    public void clearCache() {
-        cacheW.clear();
-    }
-
     private byte[] findInCache(final String name) {
         final WeakReference<byte[]> weakReference = cacheW.get(name);
         if (weakReference != null) {
@@ -171,11 +166,7 @@ public class ReferenceSource implements CRAMReferenceSource {
         {
             if (Defaults.USE_CRAM_REF_DOWNLOAD) { // try to fetch sequence by md5:
                 if (md5 != null) {
-                    try {
-                        bases = findBasesByMD5(md5.toLowerCase());
-                    } catch (final Exception e) {
-                        throw new RuntimeException(e);
-                    }
+                    bases = findBasesByMD5(md5.toLowerCase());
                 }
                 if (bases != null) {
                     return addToCache(md5, bases);
@@ -187,7 +178,7 @@ public class ReferenceSource implements CRAMReferenceSource {
         return null;
     }
 
-    byte[] findBasesByName(final String name, final boolean tryVariants) {
+    private byte[] findBasesByName(final String name, final boolean tryVariants) {
         if (rsFile == null || !rsFile.isIndexed())
             return null;
 
@@ -214,21 +205,18 @@ public class ReferenceSource implements CRAMReferenceSource {
         return null;
     }
 
-    byte[] findBasesByMD5(final String md5) throws
-            IOException {
+    private byte[] findBasesByMD5(final String md5) {
         final String url = String.format(Defaults.EBI_REFERENCE_SERVICE_URL_MASK, md5);
 
         for (int i = 0; i < downloadTriesBeforeFailing; i++) {
-            final InputStream is = new URL(url).openStream();
-            if (is == null)
-                return null;
+            try (final InputStream is = new URL(url).openStream()) {
+                if (is == null)
+                    return null;
 
-            log.debug("Downloading reference sequence: " + url);
-            final byte[] data = InputStreamUtils.readFully(is);
-            log.debug("Downloaded " + data.length + " bytes for md5 " + md5);
-            is.close();
+                log.info("Downloading reference sequence: " + url);
+                final byte[] data = InputStreamUtils.readFully(is);
+                log.info("Downloaded " + data.length + " bytes for md5 " + md5);
 
-            try {
                 final String downloadedMD5 = SequenceUtil.calculateMD5String(data);
                 if (md5.equals(downloadedMD5)) {
                     return data;
@@ -238,18 +226,19 @@ public class ReferenceSource implements CRAMReferenceSource {
                                     md5, downloadedMD5);
                     log.error(message);
                 }
-            } catch (final NoSuchAlgorithmException e) {
+            }
+            catch (final IOException e) {
                 throw new RuntimeException(e);
             }
         }
-        throw new RuntimeException("Giving up on downloading sequence for md5 "
+        throw new GaveUpException("Giving up on downloading sequence for md5 "
                 + md5);
     }
 
     private static final Pattern chrPattern = Pattern.compile("chr.*",
             Pattern.CASE_INSENSITIVE);
 
-    List<String> getVariants(final String name) {
+    private List<String> getVariants(final String name) {
         final List<String> variants = new ArrayList<>();
 
         if (name.equals("M"))
