@@ -25,12 +25,10 @@ package htsjdk.samtools.liftover;
 
 import htsjdk.samtools.SAMException;
 import htsjdk.samtools.SAMSequenceDictionary;
-import htsjdk.samtools.util.IOUtil;
-import htsjdk.samtools.util.Interval;
-import htsjdk.samtools.util.Log;
-import htsjdk.samtools.util.OverlapDetector;
+import htsjdk.samtools.util.*;
 
 import java.io.File;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -39,6 +37,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+
 /**
  * Java port of UCSC liftOver.  Only the most basic liftOver functionality is implemented.
  * Internally coordinates are 0-based, half-open. The API is standard Picard 1-based, inclusive.
@@ -84,10 +83,19 @@ public class LiftOver {
     /**
      * Load UCSC chain file in order to lift over Intervals.
      */
-    public LiftOver(File chainFile) {
-        IOUtil.assertFileIsReadable(chainFile);
-        chains = Chain.loadChains(chainFile);
+    public LiftOver(File chainFile){
+        this(Chain.loadChains(chainFile));
+    }
 
+    /**
+     * Load UCSC chain file in order to lift over Intervals.
+     */
+    public LiftOver(InputStream chainFileInputStream, String sourceName) {
+        this(Chain.loadChains(new BufferedLineReader(chainFileInputStream), sourceName));
+    }
+
+    private LiftOver(OverlapDetector<Chain> chains) {
+        this.chains = chains;
         for (final Chain chain : this.chains.getAll()) {
             final String from = chain.fromSequenceName;
             final String to   = chain.toSequenceName;
@@ -346,7 +354,7 @@ public class LiftOver {
                 // Matched a chain, but entirely within a gap.
                 return fromInterval.toString() + " (len " + fromInterval.length() + ")=>null using chain " + chainId;
             }
-            final String strand = toInterval.isNegativeStrand()? "-": "+";
+            final String strand = toInterval.getStrand().encode();
             return fromInterval.toString() + " (len " + fromInterval.length() + ")=>" + toInterval + "(" + strand
                     + ") using chain " + chainId + " ; pct matched " + percentLiftedOver;
         }
