@@ -23,15 +23,12 @@
  */
 package htsjdk.samtools.util;
 
-
 import htsjdk.samtools.Defaults;
 import htsjdk.samtools.SAMException;
-import htsjdk.samtools.SamStreams;
 import htsjdk.samtools.seekablestream.SeekableBufferedStream;
 import htsjdk.samtools.seekablestream.SeekableFileStream;
 import htsjdk.samtools.seekablestream.SeekableHTTPStream;
 import htsjdk.samtools.seekablestream.SeekableStream;
-import htsjdk.tribble.Tribble;
 import htsjdk.samtools.util.nio.DeleteOnExitPathHook;
 
 import java.io.BufferedInputStream;
@@ -57,15 +54,19 @@ import java.net.URL;
 import java.nio.charset.Charset;
 import java.nio.file.FileSystemNotFoundException;
 import java.nio.file.FileSystems;
+import java.nio.file.FileVisitResult;
 import java.nio.file.Files;
+import java.nio.file.OpenOption;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.nio.file.SimpleFileVisitor;
+import java.nio.file.StandardOpenOption;
+import java.nio.file.attribute.BasicFileAttributes;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Scanner;
@@ -91,27 +92,75 @@ public class IOUtil {
     public static final long TWO_GBS = 2 * ONE_GB;
     public static final long FIVE_GBS = 5 * ONE_GB;
 
-    public static final String VCF_FILE_EXTENSION = ".vcf";
-    public static final String VCF_INDEX_EXTENSION = Tribble.STANDARD_INDEX_EXTENSION;
-
-    public static final String BCF_FILE_EXTENSION = ".bcf";
-    public static final String COMPRESSED_VCF_FILE_EXTENSION = ".vcf.gz";
-    public static final String COMPRESSED_VCF_INDEX_EXTENSION = ".tbi";
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#VCF} instead.
+     */
+    @Deprecated
+    public static final String VCF_FILE_EXTENSION = FileExtensions.VCF;
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#VCF_INDEX} instead.
+     */
+    @Deprecated
+    public static final String VCF_INDEX_EXTENSION = FileExtensions.VCF_INDEX;
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#BCF} instead.
+     */
+    @Deprecated
+    public static final String BCF_FILE_EXTENSION = FileExtensions.BCF;
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#COMPRESSED_VCF} instead.
+     */
+    @Deprecated
+    public static final String COMPRESSED_VCF_FILE_EXTENSION = FileExtensions.COMPRESSED_VCF;
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#COMPRESSED_VCF_INDEX} instead.
+     */
+    @Deprecated
+    public static final String COMPRESSED_VCF_INDEX_EXTENSION = FileExtensions.COMPRESSED_VCF_INDEX;
 
     /** Possible extensions for VCF files and related formats. */
-    public static final String[] VCF_EXTENSIONS = {VCF_FILE_EXTENSION, COMPRESSED_VCF_FILE_EXTENSION, BCF_FILE_EXTENSION};
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#VCF_LIST} instead.
+     */
+    @Deprecated
+    public static final List<String> VCF_EXTENSIONS_LIST = FileExtensions.VCF_LIST;
 
-    public static final String INTERVAL_LIST_FILE_EXTENSION = IntervalList.INTERVAL_LIST_FILE_EXTENSION;
+    /**
+     * Possible extensions for VCF files and related formats.
+     * @deprecated since June 2019 Use {@link FileExtensions#VCF_LIST} instead.
+     */
+    @Deprecated
+    public static final String[] VCF_EXTENSIONS = FileExtensions.VCF_LIST.toArray(new String[0]);
 
-    public static final String SAM_FILE_EXTENSION = ".sam";
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#INTERVAL_LIST} instead.
+     */
+    @Deprecated
+    public static final String INTERVAL_LIST_FILE_EXTENSION = FileExtensions.INTERVAL_LIST;
 
-    public static final String DICT_FILE_EXTENSION = ".dict";
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#SAM} instead.
+     */
+    @Deprecated
+    public static final String SAM_FILE_EXTENSION = FileExtensions.SAM;
 
-    public static final Set<String> BLOCK_COMPRESSED_EXTENSIONS = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(".gz", ".gzip", ".bgz", ".bgzf")));
+    /**
+     * @deprecated since June 2019 Use {@link FileExtensions#DICT} instead.
+     */
+    @Deprecated
+    public static final String DICT_FILE_EXTENSION = FileExtensions.DICT;
+
+    /**
+     * @deprecated Use since June 2019 {@link FileExtensions#BLOCK_COMPRESSED} instead.
+     */
+    @Deprecated
+    public static final Set<String> BLOCK_COMPRESSED_EXTENSIONS = FileExtensions.BLOCK_COMPRESSED;
 
     /** number of bytes that will be read for the GZIP-header in the function {@link #isGZIPInputStream(InputStream)} */
     public static final int GZIP_HEADER_READ_LENGTH = 8000;
 
+    private static final OpenOption[] EMPTY_OPEN_OPTIONS = new OpenOption[0];
+
     private static int compressionLevel = Defaults.COMPRESSION_LEVEL;
     /**
      * Sets the GZip compression level for subsequent GZIPOutputStream object creation.
@@ -252,16 +301,33 @@ public class IOUtil {
     }
 
     public static void deletePaths(final Path... paths) {
-        deletePaths(Arrays.asList(paths));
+        for(Path path: paths){
+            deletePath(path);
+        }
     }
 
+    /**
+     * Iterate through Paths and delete each one.
+     * Note: Path is itself an Iterable<Path>.  This method special cases that and deletes the single Path rather than
+     * Iterating the Path for targets to delete.
+     * @param paths an iterable of Paths to delete
+     */
     public static void deletePaths(final Iterable<Path> paths) {
-        for (final Path p : paths) {
-            try {
-                Files.delete(p);
-            } catch (IOException e) {
-                System.err.println("Could not delete file " + p);
-            }
+        //Path is itself an Iterable<Path> which causes very confusing behavior if we don't explicitly check here.
+        if( paths instanceof Path){
+            deletePath((Path)paths);
+        }
+        paths.forEach(IOUtil::deletePath);
+    }
+
+    /**
+     * Attempt to delete a single path and log an error if it is not deleted.
+     */
+    public static void deletePath(Path path){
+        try {
+            Files.delete(path);
+        } catch (IOException e) {
+            System.err.println("Could not delete file " + path);
         }
     }
 
@@ -625,8 +691,7 @@ public class IOUtil {
     public static InputStream openFileForReading(final Path path) {
 
         try {
-            if (path.getFileName().toString().endsWith(".gz") ||
-                path.getFileName().toString().endsWith(".bfq"))  {
+            if (hasGzipFileExtension(path))  {
                 return openGzipFileForReading(path);
             }
             else {
@@ -672,30 +737,46 @@ public class IOUtil {
      * @return the output stream to write to
      */
     public static OutputStream openFileForWriting(final File file) {
-        return openFileForWriting(file, false);
+        return openFileForWriting(toPath(file));
     }
 
     /**
-     * Opens a file for writing
+     * Opens a file for writing, gzip it if it ends with ".gz" or "bfq"
      *
      * @param file  the file to write to
      * @param append    whether to append to the file if it already exists (we overwrite it if false)
      * @return the output stream to write to
      */
     public static OutputStream openFileForWriting(final File file, final boolean append) {
+        return openFileForWriting(toPath(file), getAppendOpenOption(append));
+    }
 
+    /**
+     * Opens a file for writing, gzip it if it ends with ".gz" or "bfq"
+     *
+     * @param path  the file to write to
+     * @param openOptions options to use when opening the file
+     * @return the output stream to write to
+     */
+    public static OutputStream openFileForWriting(final Path path, OpenOption... openOptions) {
         try {
-            if (file.getName().endsWith(".gz") ||
-                file.getName().endsWith(".bfq")) {
-                return openGzipFileForWriting(file, append);
-            }
-            else {
-                return new FileOutputStream(file, append);
+            if (hasGzipFileExtension(path)) {
+                return openGzipFileForWriting(path, openOptions);
+            } else {
+                return Files.newOutputStream(path, openOptions);
             }
+        } catch (final IOException ioe) {
+            throw new SAMException("Error opening file for writing: " + path.toUri().toString(), ioe);
         }
-        catch (IOException ioe) {
-            throw new SAMException("Error opening file for writing: " + file.getName(), ioe);
-        }
+    }
+
+    /**
+     * check if the file name ends with .gz, .gzip, or .bfq
+     */
+    public static boolean hasGzipFileExtension(Path path) {
+        final List<String> gzippedEndings = Arrays.asList(".gz", ".gzip", ".bfq");
+        final String fileName = path.getFileName().toString();
+        return gzippedEndings.stream().anyMatch(fileName::endsWith);
     }
 
     /**
@@ -705,19 +786,32 @@ public class IOUtil {
         return new BufferedWriter(new OutputStreamWriter(openFileForWriting(file, append)), Defaults.NON_ZERO_BUFFER_SIZE);
     }
 
+    /**
+     * Preferred over PrintStream and PrintWriter because an exception is thrown on I/O error
+     */
+    public static BufferedWriter openFileForBufferedWriting(final Path path, final OpenOption ... openOptions) {
+        return new BufferedWriter(new OutputStreamWriter(openFileForWriting(path, openOptions)), Defaults.NON_ZERO_BUFFER_SIZE);
+    }
+
     /**
      * Preferred over PrintStream and PrintWriter because an exception is thrown on I/O error
      */
     public static BufferedWriter openFileForBufferedWriting(final File file) {
-        return openFileForBufferedWriting(file, false);
+        return openFileForBufferedWriting(IOUtil.toPath(file));
     }
 
     /**
      * Preferred over PrintStream and PrintWriter because an exception is thrown on I/O error
      */
     public static BufferedWriter openFileForBufferedUtf8Writing(final File file) {
-        return new BufferedWriter(new OutputStreamWriter(openFileForWriting(file), Charset.forName("UTF-8")),
-                Defaults.NON_ZERO_BUFFER_SIZE);
+        return openFileForBufferedUtf8Writing(IOUtil.toPath(file));
+    }
+
+    /**
+     * Preferred over PrintStream and PrintWriter because an exception is thrown on I/O error
+     */
+    public static BufferedWriter openFileForBufferedUtf8Writing(final Path path) {
+        return new BufferedWriter(new OutputStreamWriter(openFileForWriting(path), Charset.forName("UTF-8")), Defaults.NON_ZERO_BUFFER_SIZE);
     }
 
     /**
@@ -738,23 +832,42 @@ public class IOUtil {
      * @return the output stream to write to
      */
     public static OutputStream openGzipFileForWriting(final File file, final boolean append) {
+        return openGzipFileForWriting(IOUtil.toPath(file), getAppendOpenOption(append));
+    }
 
+    /**
+     * converts a boolean into an array containing either the append option or nothing
+     */
+    private static OpenOption[] getAppendOpenOption(boolean append) {
+        return append ? new OpenOption[]{StandardOpenOption.APPEND} : EMPTY_OPEN_OPTIONS;
+    }
+
+    /**
+     * Opens a GZIP encoded file for writing
+     *
+     * @param path the file to write to
+     * @param openOptions options to control how the file is opened
+     * @return the output stream to write to
+     */
+    public static OutputStream openGzipFileForWriting(final Path path, final OpenOption ... openOptions) {
         try {
+            final OutputStream out = Files.newOutputStream(path, openOptions);
             if (Defaults.BUFFER_SIZE > 0) {
-            return new CustomGzipOutputStream(new FileOutputStream(file, append),
-                                              Defaults.BUFFER_SIZE,
-                                              compressionLevel);
+                return new CustomGzipOutputStream(out, Defaults.BUFFER_SIZE, compressionLevel);
             } else {
-                return new CustomGzipOutputStream(new FileOutputStream(file, append), compressionLevel);
+                return new CustomGzipOutputStream(out, compressionLevel);
             }
-        }
-        catch (IOException ioe) {
-            throw new SAMException("Error opening file for writing: " + file.getName(), ioe);
+        } catch (final IOException ioe) {
+            throw new SAMException("Error opening file for writing: " + path.toUri().toString(), ioe);
         }
     }
 
     public static OutputStream openFileForMd5CalculatingWriting(final File file) {
-        return new Md5CalculatingOutputStream(IOUtil.openFileForWriting(file), new File(file.getAbsolutePath() + ".md5"));
+        return openFileForMd5CalculatingWriting(toPath(file));
+    }
+
+    public static OutputStream openFileForMd5CalculatingWriting(final Path file) {
+        return new Md5CalculatingOutputStream(IOUtil.openFileForWriting(file), file.resolve(".md5"));
     }
 
     /**
@@ -1031,7 +1144,7 @@ public class IOUtil {
      */
     public static List<File> unrollFiles(final Collection<File> inputs, final String... extensions) {
         Collection<Path> paths = unrollPaths(filesToPaths(inputs), extensions);
-        return paths.stream().map(p->p.toFile()).collect(Collectors.toList());
+        return paths.stream().map(Path::toFile).collect(Collectors.toList());
     }
 
     /**
@@ -1059,8 +1172,6 @@ public class IOUtil {
 
             // If the file didn't match a given extension, treat it as a list of files
             if (!matched) {
-                IOUtil.assertFileIsReadable(p);
-
                 try {
                     Files.lines(p)
                             .map(String::trim)
@@ -1071,16 +1182,16 @@ public class IOUtil {
                                             innerPath = getPath(s);
                                             stack.push(innerPath);
                                         } catch (IOException e) {
-                                            throw new IllegalArgumentException("cannot convert " + s + " to a Path.");
+                                            throw new IllegalArgumentException("cannot convert " + s + " to a Path.", e);
                                         }
                                     }
                             );
 
                 } catch (IOException e) {
-                    throw new IllegalArgumentException("had trouble reading from " + p.toUri().toString());
+                    throw new IllegalArgumentException("had trouble reading from " + p.toUri().toString(), e);
                 }
             }
-    }
+        }
 
         // Preserve input order (since we're using a stack above) for things that care
         Collections.reverse(output);
@@ -1220,7 +1331,7 @@ public class IOUtil {
     /**
      * Checks if the provided path is block-compressed (including extension).
      *
-     * <p>Note that block-compressed file extensions {@link #BLOCK_COMPRESSED_EXTENSIONS} are not
+     * <p>Note that block-compressed file extensions {@link FileExtensions#BLOCK_COMPRESSED} are not
      * checked by this method.
      *
      * @param path file to check if it is block-compressed.
@@ -1232,7 +1343,7 @@ public class IOUtil {
     }
 
     /**
-     * Checks if a file ends in one of the {@link #BLOCK_COMPRESSED_EXTENSIONS}.
+     * Checks if a file ends in one of the {@link FileExtensions#BLOCK_COMPRESSED}.
      *
      * @param fileName string name for the file. May be an HTTP/S url.
      *
@@ -1240,7 +1351,7 @@ public class IOUtil {
      */
     public static boolean hasBlockCompressedExtension (final String fileName) {
         String cleanedPath = stripQueryStringIfPathIsAnHttpUrl(fileName);
-        for (final String extension : BLOCK_COMPRESSED_EXTENSIONS) {
+        for (final String extension : FileExtensions.BLOCK_COMPRESSED) {
             if (cleanedPath.toLowerCase().endsWith(extension))
                 return true;
         }
@@ -1248,7 +1359,7 @@ public class IOUtil {
     }
 
     /**
-     * Checks if a path ends in one of the {@link #BLOCK_COMPRESSED_EXTENSIONS}.
+     * Checks if a path ends in one of the {@link FileExtensions#BLOCK_COMPRESSED}.
      *
      * @param path object to extract the name from.
      *
@@ -1259,7 +1370,7 @@ public class IOUtil {
     }
 
     /**
-     * Checks if a file ends in one of the {@link #BLOCK_COMPRESSED_EXTENSIONS}.
+     * Checks if a file ends in one of the {@link FileExtensions#BLOCK_COMPRESSED}.
      *
      * @param file object to extract the name from.
      *
@@ -1270,7 +1381,7 @@ public class IOUtil {
     }
 
     /**
-     * Checks if a file ends in one of the {@link #BLOCK_COMPRESSED_EXTENSIONS}.
+     * Checks if a file ends in one of the {@link FileExtensions#BLOCK_COMPRESSED}.
      *
      * @param uri file as an URI.
      *
@@ -1296,4 +1407,34 @@ public class IOUtil {
         }
         return path;
     }
+
+    /**
+     * Delete a directory and all files in it.
+     *
+     * @param directory The directory to be deleted (along with its subdirectories)
+     */
+    public static void recursiveDelete(final Path directory) {
+        
+        final SimpleFileVisitor<Path> simpleFileVisitor = new SimpleFileVisitor<Path>() {
+            @Override
+            public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
+                super.visitFile(file, attrs);
+                Files.deleteIfExists(file);
+                return FileVisitResult.CONTINUE;
+            }
+
+            @Override
+            public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
+                super.postVisitDirectory(dir, exc);
+                Files.deleteIfExists(dir);
+                return FileVisitResult.CONTINUE;
+            }
+        };
+
+        try {
+            Files.walkFileTree(directory, simpleFileVisitor);
+        } catch (final IOException e){
+            throw new RuntimeIOException(e);
+        }
+    }
 }
