@@ -1,5 +1,7 @@
 package htsjdk.samtools.util;
 
+import java.util.function.Supplier;
+
 /**
  * Simple utility for building an on-demand (lazy) object-initializer.
  * 
@@ -9,29 +11,36 @@ package htsjdk.samtools.util;
  * @author mccowan
  */
 public class Lazy<T> {
-    private final LazyInitializer<T> initializer;
+    private final Supplier<T> initializer;
     private boolean isInitialized = false;
     private T instance;
 
-    /** Simple cons */
-    public Lazy(final LazyInitializer<T> initializer) {
+    public Lazy(final Supplier<T> initializer) {
         this.initializer = initializer;
     }
 
     /** Returns the instance associated with this {@link Lazy}, initializing it if necessary. */
     public synchronized T get() {
         if (!isInitialized) {
-            this.instance = initializer.make();
+            this.instance = initializer.get();
             isInitialized = true;
         }
         return instance;
     }
 
-    /** Describes how to build the instance of the lazy object. */
+    /** Describes how to build the instance of the lazy object.
+     * @deprecated since 1/2017 use a {@link Supplier} instead
+     * */
     @FunctionalInterface
-    public interface LazyInitializer<T> {
+    @Deprecated
+    public interface LazyInitializer<T> extends Supplier<T> {
         /** Returns the desired object instance. */
         T make();
+
+        @Override
+        default T get(){
+            return make();
+        }
     }
 
     public boolean isInitialized() {
