@@ -23,16 +23,10 @@
  */
 package htsjdk.samtools.util;
 
-import htsjdk.samtools.AlignmentBlock;
-import htsjdk.samtools.Cigar;
-import htsjdk.samtools.CigarElement;
-import htsjdk.samtools.CigarOperator;
-import htsjdk.samtools.SAMException;
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMSequenceDictionary;
-import htsjdk.samtools.SAMSequenceRecord;
-import htsjdk.samtools.SAMTag;
+import htsjdk.samtools.*;
 import htsjdk.samtools.fastq.FastqConstants;
+import htsjdk.tribble.annotation.Strand;
+import htsjdk.utils.ValidationUtils;
 
 import java.io.File;
 import java.math.BigInteger;
@@ -41,6 +35,7 @@ import java.security.NoSuchAlgorithmException;
 import java.util.Arrays;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Random;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -58,15 +53,17 @@ public class SequenceUtil {
      */
     private static final byte[] BAM_READ_BASE_SET = "=ABCDGHKMNRSTVWY".getBytes();
 
+    private static final int BASES_ARRAY_LENGTH = 127;
+    private static final int SHIFT_TO_LOWER_CASE = a - A;
     /**
      * A lookup table to find a corresponding BAM read base.
      */
-    private static final byte[] bamReadBaseLookup = new byte[127];
+    private static final byte[] bamReadBaseLookup = new byte[BASES_ARRAY_LENGTH];
     static {
         Arrays.fill(bamReadBaseLookup, N);
         for (final byte base: BAM_READ_BASE_SET) {
             bamReadBaseLookup[base] = base;
-            bamReadBaseLookup[base + 32] = base;
+            bamReadBaseLookup[base + SHIFT_TO_LOWER_CASE] = base;
         }
     }
 
@@ -75,7 +72,7 @@ public class SequenceUtil {
     private static final byte G_MASK = 4;
     private static final byte T_MASK = 8;
 
-    private static final byte[] bases = new byte[127];
+    private static final byte[] bases = new byte[BASES_ARRAY_LENGTH];
     private static final byte NON_IUPAC_CODE = 0;
     /*
      * Definition of IUPAC codes:
@@ -100,7 +97,7 @@ public class SequenceUtil {
         bases['N'] = A_MASK | C_MASK | G_MASK | T_MASK;
         // Also store the bases in lower case
         for (int i = 'A'; i <= 'Z'; i++) {
-            bases[(byte) i + 32] = bases[(byte) i];
+            bases[(byte) i + SHIFT_TO_LOWER_CASE] = bases[(byte) i];
         }
         bases['.'] = A_MASK | C_MASK | G_MASK | T_MASK;
     };
@@ -125,6 +122,12 @@ public class SequenceUtil {
      * without considering the set relationships between ambiguous codes.
      */
     public static boolean basesEqual(final byte lhs, final byte rhs) {
+        if (lhs < 0 || lhs >= BASES_ARRAY_LENGTH) {
+            return false;
+        }
+        if (rhs < 0 || rhs >= BASES_ARRAY_LENGTH) {
+            return false;
+        }
         return (bases[lhs] == bases[rhs]);
     }
 
@@ -139,6 +142,13 @@ public class SequenceUtil {
      * Since the comparison is directional, make sure to pass read / ref codes in correct order.
      */
     public static boolean readBaseMatchesRefBaseWithAmbiguity(final byte readBase, final byte refBase) {
+        if (readBase < 0 || readBase >= BASES_ARRAY_LENGTH) {
+            return false;
+        }
+        if (refBase < 0 || refBase >= BASES_ARRAY_LENGTH) {
+            return false;
+        }
+
         return (bases[readBase] & bases[refBase]) == bases[readBase];
     }
 
@@ -175,15 +185,19 @@ public class SequenceUtil {
 
     /** Checks if the given base is a IUPAC code */
     public static boolean isIUPAC(final byte base) {
+        if (base >= BASES_ARRAY_LENGTH || base < 0) {
+            return false;
+        }
         return bases[base] != NON_IUPAC_CODE;
     }
 
     /** Calculates the fraction of bases that are G/C in the sequence */
     public static double calculateGc(final byte[] bases) {
         int gcs = 0;
-        for (int i = 0; i < bases.length; ++i) {
-            final byte b = bases[i];
-            if (b == 'C' || b == 'G' || b == 'c' || b == 'g') ++gcs;
+        for (final byte b : bases) {
+            if (b == 'C' || b == 'G' || b == 'c' || b == 'g') {
+                ++gcs;
+            }
         }
 
         return gcs / (double) bases.length;
@@ -240,10 +254,10 @@ public class SequenceUtil {
             }
             for (int i = 0; i < sizeToTest; ++i) {
                 if (!s1.get(i).isSameSequence(s2.get(i))) {
-                    String s1Attrs = "";
+                    StringBuilder s1Attrs = new StringBuilder();
                     for (final java.util.Map.Entry<String, String> entry : s1.get(i)
                             .getAttributes()) {
-                        s1Attrs += "/" + entry.getKey() + "=" + entry.getValue();
+                        s1Attrs.append("/").append(entry.getKey()).append("=").append(entry.getValue());
                     }
                     String s2Attrs = "";
                     for (final java.util.Map.Entry<String, String> entry : s2.get(i)
@@ -898,16 +912,25 @@ public class SequenceUtil {
         }
     }
 
-    public static String calculateMD5String(final byte[] data)
-            throws NoSuchAlgorithmException {
+    public static String calculateMD5String(final byte[] data) {
         return SequenceUtil.calculateMD5String(data, 0, data.length);
     }
 
     public static String calculateMD5String(final byte[] data, final int offset, final int len) {
         final byte[] digest = calculateMD5(data, offset, len);
+        return md5DigestToString(digest);
+    }
+
+    /**
+     * Convets the result of an md5Digest to a string
+     * @param digest digest that needs to be converted to a string
+     * @return string representing the md5
+     */
+    public static String md5DigestToString(final byte[] digest) {
         return String.format("%032x", new BigInteger(1, digest));
     }
 
+
     public static byte[] calculateMD5(final byte[] data, final int offset, final int len) {
         final MessageDigest md5_MessageDigest;
         try {
@@ -1062,4 +1085,37 @@ public class SequenceUtil {
 
         return readName;
     }
+
+    /**
+     *  Returns an array of bytes containing random DNA bases
+     *
+     * @param random A {@link Random} object to use for drawing randomw bases
+     * @param length How many bases to return.
+     * @return an array of random DNA bases of the requested length.
+     */
+    static public byte[] getRandomBases(Random random, final int length) {
+        ValidationUtils.validateArg(length >= 0, "length must be non-negative");
+        final byte[] bases = new byte[length];
+        getRandomBases(random, length, bases);
+        return bases;
+    }
+
+    /**
+     * Fills an array of bytes with random DNA bases. Will overwrite
+     * first {@code length} byte with new bases. if {@code length} is
+     * less than the size of the input array, the remaining bases will
+     * not be modified.
+     *
+     * @param random A {@link Random} object to use for drawing random bases
+     * @param length How many bases to return.
+     * @param bases  Array to use for bases (from index 0)
+     */
+    static public void getRandomBases(Random random, final int length, final byte[] bases) {
+        ValidationUtils.validateArg(length >= 0, "length must be non-negative");
+        ValidationUtils.validateArg(length <= bases.length, "length must no larger than size of input array");
+
+        for (int i = 0; i < length; ++i) {
+            bases[i] = VALID_BASES_UPPER[random.nextInt(VALID_BASES_UPPER.length)];
+        }
+    }
 }
