@@ -26,9 +26,17 @@ package htsjdk.samtools.util;
 import htsjdk.samtools.SAMException;
 
 import java.io.*;
+import java.nio.file.FileVisitResult;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.SimpleFileVisitor;
+import java.nio.file.attribute.BasicFileAttributes;
 
 public class TestUtil {
 
+    public static final int RANDOM_SEED = 42;
+
+
     /**
      * Base url where all test files for http tests are found
      */
@@ -57,21 +65,6 @@ public class TestUtil {
         return getTempDirectory(prefix, suffix);
     }
 
-        /**
-         * Little test utility to help tests that create multiple levels of subdirectories
-         * clean up after themselves.
-         *
-         * @param directory The directory to be deleted (along with its subdirectories)
-         */
-    public static void recursiveDelete(final File directory) {
-        for (final File f : directory.listFiles()) {
-            if (f.isDirectory()) {
-                recursiveDelete(f);
-            }
-            f.delete();
-        }
-    }
-
     /**
      * Serialize and Deserialize an object
      * Useful for testing if serialization is correctly handled for a class.
@@ -95,4 +88,20 @@ public class TestUtil {
         in.close();
         return result;
     }
+
+    /**
+     * Little test utility to help tests that create multiple levels of subdirectories
+     * clean up after themselves.
+     *
+     * @param directory The directory to be deleted (along with its subdirectories)
+     * @deprecated Since 3/19, prefer {@link IOUtil#recursiveDelete(Path)}
+     */
+    @Deprecated
+    public static void recursiveDelete(final File directory) {
+        try {
+            IOUtil.recursiveDelete(directory.toPath());
+        } catch (RuntimeIOException e) {
+            // bury exception
+        }
+    }
 }
