@@ -25,7 +25,7 @@ import java.util.TreeMap;
 public class VCFEncoder {
 
     /**
-     * The encoding used for VCF files: ISO-8859-1
+     * The encoding used for VCF files: ISO-8859-1. When writing VCF4.3 is implemented, this should change to UTF-8.
      */
     public static final Charset VCF_CHARSET = Charset.forName("ISO-8859-1");
     private static final String QUAL_FORMAT_STRING = "%.2f";
@@ -44,7 +44,9 @@ public class VCFEncoder {
      * allowing missing fields in the header.
      */
     public VCFEncoder(final VCFHeader header, final boolean allowMissingFieldsInHeader, final boolean outputTrailingFormatFields) {
-        if (header == null) throw new NullPointerException("The VCF header must not be null.");
+        if (header == null) {
+            throw new NullPointerException("The VCF header must not be null.");
+        }
         this.header = header;
         this.allowMissingFieldsInHeader = allowMissingFieldsInHeader;
         this.outputTrailingFormatFields = outputTrailingFormatFields;
@@ -128,8 +130,11 @@ public class VCFEncoder {
         vcfOutput.append(VCFConstants.FIELD_SEPARATOR);
 
         // QUAL
-        if ( ! context.hasLog10PError()) vcfOutput.append(VCFConstants.MISSING_VALUE_v4);
-        else vcfOutput.append(formatQualValue(context.getPhredScaledQual()));
+        if ( !context.hasLog10PError()) {
+            vcfOutput.append(VCFConstants.MISSING_VALUE_v4);
+        } else {
+            vcfOutput.append(formatQualValue(context.getPhredScaledQual()));
+        }
         vcfOutput.append(VCFConstants.FIELD_SEPARATOR)
                 // FILTER
                 .append(getFilterString(context)).append(VCFConstants.FIELD_SEPARATOR);
@@ -137,11 +142,14 @@ public class VCFEncoder {
         // INFO
         final Map<String, String> infoFields = new TreeMap<>();
         for (final Map.Entry<String, Object> field : context.getAttributes().entrySet()) {
-            if (!this.header.hasInfoLine(field.getKey()))
+            if (!this.header.hasInfoLine(field.getKey())) {
                 fieldIsMissingFromHeaderError(context, field.getKey(), "INFO");
+            }
 
             final String outputValue = formatVCFField(field.getValue());
-            if (outputValue != null) infoFields.put(field.getKey(), outputValue);
+            if (outputValue != null) {
+                infoFields.put(field.getKey(), outputValue);
+            }
         }
         writeInfoString(infoFields, vcfOutput);
 
@@ -152,11 +160,12 @@ public class VCFEncoder {
             vcfOutput.append(((LazyGenotypesContext) gc).getUnparsedGenotypeData().toString());
         } else {
             final List<String> genotypeAttributeKeys = context.calcVCFGenotypeKeys(this.header);
-            if ( ! genotypeAttributeKeys.isEmpty()) {
-                for (final String format : genotypeAttributeKeys)
-                    if ( ! this.header.hasFormatLine(format))
+            if ( !genotypeAttributeKeys.isEmpty()) {
+                for (final String format : genotypeAttributeKeys) {
+                    if (!this.header.hasFormatLine(format)) {
                         fieldIsMissingFromHeaderError(context, format, "FORMAT");
-
+                    }
+                }
                 final String genotypeFormatString = ParsingUtils.join(VCFConstants.GENOTYPE_FIELD_SEPARATOR, genotypeAttributeKeys);
 
                 vcfOutput.append(VCFConstants.FIELD_SEPARATOR);
@@ -166,8 +175,6 @@ public class VCFEncoder {
                 appendGenotypeData(context, alleleStrings, genotypeAttributeKeys, vcfOutput);
             }
         }
-
-
     }
 
     VCFHeader getVCFHeader() {
@@ -181,52 +188,60 @@ public class VCFEncoder {
     private String getFilterString(final VariantContext vc) {
         if (vc.isFiltered()) {
             for (final String filter : vc.getFilters()) {
-                if (!this.header.hasFilterLine(filter)) fieldIsMissingFromHeaderError(vc, filter, "FILTER");
+                if (!this.header.hasFilterLine(filter)) {
+                    fieldIsMissingFromHeaderError(vc, filter, "FILTER");
+                }
             }
 
             return ParsingUtils.join(";", ParsingUtils.sortList(vc.getFilters()));
-        } else if (vc.filtersWereApplied()) return VCFConstants.PASSES_FILTERS_v4;
-        else return VCFConstants.UNFILTERED;
+        } else {
+            return vc.filtersWereApplied() ? VCFConstants.PASSES_FILTERS_v4 : VCFConstants.UNFILTERED;
+        }
     }
 
     private static String formatQualValue(final double qual) {
         String s = String.format(QUAL_FORMAT_STRING, qual);
-        if (s.endsWith(QUAL_FORMAT_EXTENSION_TO_TRIM))
+        if (s.endsWith(QUAL_FORMAT_EXTENSION_TO_TRIM)) {
             s = s.substring(0, s.length() - QUAL_FORMAT_EXTENSION_TO_TRIM.length());
+        }
         return s;
     }
 
     private void fieldIsMissingFromHeaderError(final VariantContext vc, final String id, final String field) {
-        if (!allowMissingFieldsInHeader)
+        if (!allowMissingFieldsInHeader) {
             throw new IllegalStateException("Key " + id + " found in VariantContext field " + field
-                                                    + " at " + vc.getContig() + ":" + vc.getStart()
-                                                    + " but this key isn't defined in the VCFHeader.  We require all VCFs to have"
-                                                    + " complete VCF headers by default.");
+                    + " at " + vc.getContig() + ":" + vc.getStart()
+                    + " but this key isn't defined in the VCFHeader.  We require all VCFs to have"
+                    + " complete VCF headers by default.");
+        }
     }
 
     @SuppressWarnings("rawtypes")
     String formatVCFField(final Object val) {
         final String result;
-        if ( val == null )
+        if (val == null) {
             result = VCFConstants.MISSING_VALUE_v4;
-        else if ( val instanceof Double )
+        } else if (val instanceof Double) {
             result = formatVCFDouble((Double) val);
-        else if ( val instanceof Boolean )
-            result = (Boolean)val ? "" : null; // empty string for true, null for false
-        else if ( val instanceof List ) {
-            result = formatVCFField(((List)val).toArray());
-        } else if ( val.getClass().isArray() ) {
+        } else if (val instanceof Boolean) {
+            result = (Boolean) val ? "" : null; // empty string for true, null for false
+        } else if (val instanceof List) {
+            result = formatVCFField(((List) val).toArray());
+        } else if (val.getClass().isArray()) {
             final int length = Array.getLength(val);
-            if ( length == 0 )
+            if (length == 0) {
                 return formatVCFField(null);
-            final StringBuilder sb = new StringBuilder(formatVCFField(Array.get(val, 0)));
-            for ( int i = 1; i < length; i++) {
+            }
+            final StringBuilder sb = new StringBuilder(
+                formatVCFField(Array.get(val, 0)));
+            for (int i = 1; i < length; i++) {
                 sb.append(',');
                 sb.append(formatVCFField(Array.get(val, i)));
             }
             result = sb.toString();
-        } else
+        } else {
             result = val.toString();
+        }
 
         return result;
     }
@@ -245,9 +260,9 @@ public class VCFEncoder {
         final String format;
         if (d < 1) {
             if (d < 0.01) {
-                if (Math.abs(d) >= 1e-20)
+                if (Math.abs(d) >= 1e-20) {
                     format = "%.3e";
-                else {
+                } else {
                     // return a zero format
                     return "0.00";
                 }
@@ -299,7 +314,9 @@ public class VCFEncoder {
             vcfoutput.append(VCFConstants.FIELD_SEPARATOR);
 
             Genotype g = vc.getGenotype(sample);
-            if (g == null) g = GenotypeBuilder.createMissing(sample, ploidy);
+            if (g == null) {
+                g = GenotypeBuilder.createMissing(sample, ploidy);
+            }
 
             final List<String> attrs = new ArrayList<>(genotypeFormatKeys.size());
             for (final String field : genotypeFormatKeys) {
@@ -323,11 +340,11 @@ public class VCFEncoder {
                         final IntGenotypeFieldAccessors.Accessor accessor = GENOTYPE_FIELD_ACCESSORS.getAccessor(field);
                         if (accessor != null) {
                             final int[] intValues = accessor.getValues(g);
-                            if (intValues == null)
+                            if (intValues == null) {
                                 outputValue = VCFConstants.MISSING_VALUE_v4;
-                            else if (intValues.length == 1) // fast path
+                            } else if (intValues.length == 1) { // fast path
                                 outputValue = Integer.toString(intValues[0]);
-                            else {
+                            } else {
                                 final StringBuilder sb = new StringBuilder();
                                 sb.append(intValues[0]);
                                 for (int i = 1; i < intValues.length; i++) {
@@ -338,37 +355,24 @@ public class VCFEncoder {
                             }
                         } else {
                             Object val = g.hasExtendedAttribute(field) ? g.getExtendedAttribute(field) : VCFConstants.MISSING_VALUE_v4;
-
-                            final VCFFormatHeaderLine metaData = this.header.getFormatHeaderLine(field);
-                            if (metaData != null) {
-                                final int numInFormatField = metaData.getCount(vc);
-                                if (numInFormatField > 1 && val.equals(VCFConstants.MISSING_VALUE_v4)) {
-                                    // If we have a missing field but multiple values are expected, we need to construct a new string with all fields.
-                                    // For example, if Number=2, the string has to be ".,."
-                                    final StringBuilder sb = new StringBuilder(VCFConstants.MISSING_VALUE_v4);
-                                    for (int i = 1; i < numInFormatField; i++) {
-                                        sb.append(',');
-                                        sb.append(VCFConstants.MISSING_VALUE_v4);
-                                    }
-                                    val = sb.toString();
-                                }
-                            }
-
-                            // assume that if key is absent, then the given string encoding suffices
                             outputValue = formatVCFField(val);
                         }
                     }
 
-                    if (outputValue != null)
+                    if (outputValue != null) {
                         attrs.add(outputValue);
+                    }
                 }
             }
 
             // strip off trailing missing values
             if (!outputTrailingFormatFields) {
                 for (int i = attrs.size() - 1; i >= 0; i--) {
-                    if (isMissingValue(attrs.get(i))) attrs.remove(i);
-                    else break;
+                    if (isMissingValue(attrs.get(i))) {
+                        attrs.remove(i);
+                    } else {
+                        break;
+                    }
                 }
             }
 
@@ -392,8 +396,11 @@ public class VCFEncoder {
 
         boolean isFirst = true;
         for (final Map.Entry<String, String> entry : infoFields.entrySet()) {
-            if (isFirst) isFirst = false;
-            else vcfoutput.append(VCFConstants.INFO_FIELD_SEPARATOR);
+            if (isFirst) {
+                isFirst = false;
+            } else {
+                vcfoutput.append(VCFConstants.INFO_FIELD_SEPARATOR);
+            }
 
             vcfoutput.append(entry.getKey());
 
