@@ -5,7 +5,6 @@ import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
-import java.io.IOException;
 
 public class EnaRefServiceTest extends HtsjdkTest {
 
@@ -16,7 +15,7 @@ public class EnaRefServiceTest extends HtsjdkTest {
     }
 
     @Test(dataProvider = "testEnaRefServiceData")
-    public void testEnaRefServiceData(final String md5) throws IOException, EnaRefService.GaveUpException {
+    public void testEnaRefServiceData(final String md5) throws GaveUpException {
         Assert.assertNotNull(new EnaRefService().getSequence(md5));
     }
 }
