@@ -1,6 +1,7 @@
 package htsjdk.samtools.seekablestream;
 
 import htsjdk.HtsjdkTest;
+import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.TestUtil;
 import org.testng.Assert;
 import org.testng.annotations.DataProvider;
@@ -8,13 +9,16 @@ import org.testng.annotations.Test;
 
 import java.io.File;
 import java.io.IOException;
+import java.net.MalformedURLException;
+import java.net.URISyntaxException;
 import java.net.URL;
+import java.nio.file.Paths;
 
 public class SeekableStreamFactoryTest extends HtsjdkTest {
     private static final File TEST_DATA_DIR = new File("src/test/resources/htsjdk/samtools");
 
     @Test
-    public void testIsFilePath() throws Exception {
+    public void testIsFilePath() {
         Assert.assertEquals(SeekableStreamFactory.isFilePath("x"), true);
         Assert.assertEquals(SeekableStreamFactory.isFilePath(""), true);
         Assert.assertEquals(SeekableStreamFactory.isFilePath("http://broadinstitute.org"), false);
@@ -23,7 +27,7 @@ public class SeekableStreamFactoryTest extends HtsjdkTest {
     }
 
     @DataProvider(name="getStreamForData")
-    public Object[][] getStreamForData() throws Exception {
+    public Object[][] getStreamForData() throws MalformedURLException {
         return new Object[][] {
                 { new File(TEST_DATA_DIR, "BAMFileIndexTest/index_test.bam").getAbsolutePath(),
                         new File(TEST_DATA_DIR, "BAMFileIndexTest/index_test.bam").getAbsolutePath() },
@@ -43,4 +47,29 @@ public class SeekableStreamFactoryTest extends HtsjdkTest {
         Assert.assertEquals(SeekableStreamFactory.getInstance().getStreamFor(path).getSource(), expectedPath);
     }
 
+    @Test
+    public void testPathWithEmbeddedSpace() throws IOException {
+        final File testBam =  new File(TEST_DATA_DIR, "BAMFileIndexTest/index_test.bam");
+
+        //create a temp dir with a space in the name and copy the test file there
+        final File tempDir = IOUtil.createTempDir("test spaces", "");
+        Assert.assertTrue(tempDir.getAbsolutePath().contains(" "));
+        tempDir.deleteOnExit();
+        final File inputBam = new File(tempDir, "index_test.bam");
+        inputBam.deleteOnExit();
+        IOUtil.copyFile(testBam, inputBam);
+
+        // make sure the input string we use is URL-encoded
+        final String inputString = Paths.get(inputBam.getAbsolutePath()).toUri().toString();
+        Assert.assertFalse(inputString.contains(" "));
+        Assert.assertTrue(inputString.contains("%20"));
+
+        try (final SeekableStream seekableStream =
+                     SeekableStreamFactory.getInstance().getStreamFor(inputString)) {
+            final int BYTES_TO_READ = 10;
+            Assert.assertEquals(seekableStream.read(new byte[BYTES_TO_READ], 0,BYTES_TO_READ), BYTES_TO_READ);
+        }
+
+    }
+
 }
