@@ -24,17 +24,22 @@
 package htsjdk.tribble.index;
 
 import com.google.common.io.Files;
+import com.google.common.jimfs.Configuration;
+import com.google.common.jimfs.Jimfs;
 import htsjdk.HtsjdkTest;
 import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.SAMSequenceRecord;
 import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.Interval;
+import htsjdk.tribble.AbstractFeatureReader;
 import htsjdk.tribble.TestUtils;
 import htsjdk.tribble.Tribble;
 import htsjdk.tribble.TribbleException;
+import htsjdk.tribble.VCFRedirectCodec;
 import htsjdk.tribble.bed.BEDCodec;
 import htsjdk.tribble.index.tabix.TabixFormat;
 import htsjdk.tribble.index.tabix.TabixIndex;
+import htsjdk.tribble.readers.LineIterator;
 import htsjdk.variant.bcf2.BCF2Codec;
 import htsjdk.variant.variantcontext.VariantContext;
 import htsjdk.variant.vcf.VCFCodec;
@@ -45,6 +50,9 @@ import org.testng.annotations.Test;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.FileSystem;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.Iterator;
 import java.util.List;
 
@@ -63,7 +71,7 @@ public class IndexFactoryTest extends HtsjdkTest {
     }
 
     @Test(dataProvider = "bedDataProvider")
-    public void testCreateLinearIndexFromBED(final File inputBEDFIle) throws Exception {
+    public void testCreateLinearIndexFromBED(final File inputBEDFIle) {
         Index index = IndexFactory.createLinearIndex(inputBEDFIle, new BEDCodec());
         String chr = "chr2";
 
@@ -78,7 +86,7 @@ public class IndexFactoryTest extends HtsjdkTest {
     }
 
     @Test(expectedExceptions = TribbleException.MalformedFeatureFile.class, dataProvider = "indexFactoryProvider")
-    public void testCreateIndexUnsorted(IndexFactory.IndexType type) throws Exception{
+    public void testCreateIndexUnsorted(IndexFactory.IndexType type) {
         final File unsortedBedFile = new File(TestUtils.DATA_DIR, "bed/unsorted.bed");
         IndexFactory.createIndex(unsortedBedFile, new BEDCodec(), type);
     }
@@ -101,15 +109,14 @@ public class IndexFactoryTest extends HtsjdkTest {
     @Test
     public void testCreateTabixIndexOnBlockCompressed() {
         // index a VCF
-        final File inputFileVcf = new File("src/test/resources/htsjdk/tribble/tabix/testTabixIndex.vcf");
+        final Path inputFileVcf = Paths.get("src/test/resources/htsjdk/tribble/tabix/testTabixIndex.vcf");
         final VCFFileReader readerVcf = new VCFFileReader(inputFileVcf, false);
         final SAMSequenceDictionary vcfDict = readerVcf.getFileHeader().getSequenceDictionary();
         final TabixIndex tabixIndexVcf =
-                IndexFactory.createTabixIndex(inputFileVcf, new VCFCodec(), TabixFormat.VCF,
-                        vcfDict);
+                IndexFactory.createTabixIndex(inputFileVcf, new VCFCodec(), TabixFormat.VCF, vcfDict);
 
         // index the same bgzipped VCF
-        final File inputFileVcfGz = new File("src/test/resources/htsjdk/tribble/tabix/testTabixIndex.vcf.gz");
+        final Path inputFileVcfGz = Paths.get("src/test/resources/htsjdk/tribble/tabix/testTabixIndex.vcf.gz");
         final VCFFileReader readerVcfGz = new VCFFileReader(inputFileVcfGz, false);
         final TabixIndex tabixIndexVcfGz =
                 IndexFactory.createTabixIndex(inputFileVcfGz, new VCFCodec(), TabixFormat.VCF,
@@ -129,6 +136,17 @@ public class IndexFactoryTest extends HtsjdkTest {
 
     }
 
+    @Test
+    public void testTabixOnNonDefaultFileSystem() throws IOException {
+        try (final FileSystem fs = Jimfs.newFileSystem("test", Configuration.unix())) {
+            final Path vcfInJimfs = TestUtils.getTribbleFileInJimfs(
+                    "src/test/resources/htsjdk/tribble/tabix/testTabixIndex.vcf.gz",
+                    null, fs);
+            // IndexFactory doesn't write the output, so we just want to make sure it doesn't throw
+            IndexFactory.createTabixIndex(vcfInJimfs, new VCFCodec(), TabixFormat.VCF, null);
+        }
+    }
+
     @DataProvider(name = "vcfDataProvider")
     public Object[][] getVCFIndexData(){
         return new Object[][] {
@@ -219,4 +237,40 @@ public class IndexFactoryTest extends HtsjdkTest {
             }
         }
     }
+
+    @DataProvider
+    public Object[][] getRedirectFiles(){
+        return new Object[][] {
+                {VCFRedirectCodec.REDIRECTING_CODEC_TEST_FILE_ROOT + "vcf.gz.redirect", IndexFactory.IndexType.TABIX},
+                {VCFRedirectCodec.REDIRECTING_CODEC_TEST_FILE_ROOT + "vcf.redirect", IndexFactory.IndexType.INTERVAL_TREE},
+                {VCFRedirectCodec.REDIRECTING_CODEC_TEST_FILE_ROOT + "vcf.redirect", IndexFactory.IndexType.LINEAR}
+        };
+    }
+
+    @Test(dataProvider = "getRedirectFiles")
+    public void testIndexRedirectedFiles(String input, IndexFactory.IndexType type) throws IOException {
+        final VCFRedirectCodec codec = new VCFRedirectCodec();
+        final File dir = IOUtil.createTempDir("redirec-test", "dir");
+        try {
+            final File tmpInput = new File(dir, new File(input).getName());
+            Files.copy(new File(input), tmpInput);
+            final File tmpDataFile = new File(codec.getPathToDataFile(tmpInput.toString()));
+            Assert.assertTrue(new File(tmpDataFile.getAbsoluteFile().getParent()).mkdir());
+            final File originalDataFile = new File(codec.getPathToDataFile(input));
+            Files.copy(originalDataFile, tmpDataFile);
+
+            try(final AbstractFeatureReader<VariantContext, LineIterator> featureReader = AbstractFeatureReader.getFeatureReader(tmpInput.getAbsolutePath(), codec, false)) {
+                Assert.assertFalse(featureReader.hasIndex());
+            }
+            final Index index = IndexFactory.createIndex(tmpInput, codec, type);
+            index.writeBasedOnFeatureFile(tmpDataFile);
+
+            try(final AbstractFeatureReader<VariantContext, LineIterator> featureReader = AbstractFeatureReader.getFeatureReader(tmpInput.getAbsolutePath(), codec)) {
+                Assert.assertTrue(featureReader.hasIndex());
+                Assert.assertEquals(featureReader.query("20",1110696,1230237).stream().count(), 2);
+            }
+        } finally {
+            IOUtil.recursiveDelete(dir.toPath());
+        }
+    }
 }
